#!/bin/bash

function flag() {
  [[ -e ../unit-data/flags/$1 ]] && return 0 || return 1
}

function flags() {
  [[ ! -d ../unit-data/flags ]] && return
  (cd ../unit-data/flags; ls "$@"*)
}

function set_flag() {
  mkdir -p ../unit-data/flags
  touch ../unit-data/flags/$1
}

function unset_flag() {
  rm -f ../unit-data/flags/$1
}

function load_dashboards_from_config() {
  if ! flag es_ready; then
    return  # ES relation not ready
  fi
  if [[ -z "$(config-get dashboards)" ]]; then
    return  # no dashboards set
  fi

  for dashboard in $(config-get dashboards); do
    if ! flag loaded_$dashboard; then
      actions/load-dashboard $dashboard
      set_flag loaded_$dashboard
    fi
  done
}

function correct_mappings_and_reindex() {
  # Nasty bug in kibana-5 that causes the ES plugin to fail like this:
  #  mapper [hits] cannot be changed from type [long] to [integer]
  # Solution is to create a template with correct mappings, and reindex:
  #  https://github.com/elastic/kibana/issues/9888
  curl -XPUT "http://localhost:9200/_template/kibana" -H 'Content-Type: application/json' -d'
  {
    "template": ".kibana-5*",
    "settings": {
      "number_of_shards": 1
    },
    "mappings": {
      "config": {
        "properties": {
          "buildNum": {
            "type": "keyword"
          }
        }
      },
      "index-pattern": {
        "properties": {
          "fieldFormatMap": {
            "type": "text"
          },
          "fields": {
            "type": "text"
          },
          "intervalName": {
            "type": "keyword"
          },
          "notExpandable": {
            "type": "boolean"
          },
          "sourceFilters": {
            "type": "text"
          },
          "timeFieldName": {
            "type": "keyword"
          },
          "title": {
            "type": "text"
          }
        }
      },
      "visualization": {
        "properties": {
          "description": {
            "type": "text"
          },
          "kibanaSavedObjectMeta": {
            "properties": {
              "searchSourceJSON": {
                "type": "text"
              }
            }
          },
          "savedSearchId": {
            "type": "keyword"
          },
          "title": {
            "type": "text"
          },
          "uiStateJSON": {
            "type": "text"
          },
          "version": {
            "type": "integer"
          },
          "visState": {
            "type": "text"
          }
        }
      },
      "search": {
        "properties": {
          "columns": {
            "type": "keyword"
          },
          "description": {
            "type": "text"
          },
          "hits": {
            "type": "integer"
          },
          "kibanaSavedObjectMeta": {
            "properties": {
              "searchSourceJSON": {
                "type": "text"
              }
            }
          },
          "sort": {
            "type": "keyword"
          },
          "title": {
            "type": "text"
          },
          "version": {
            "type": "integer"
          }
        }
      },
      "dashboard": {
        "properties": {
          "description": {
            "type": "text"
          },
          "hits": {
            "type": "integer"
          },
          "kibanaSavedObjectMeta": {
            "properties": {
              "searchSourceJSON": {
                "type": "text"
              }
            }
          },
          "optionsJSON": {
            "type": "text"
          },
          "panelsJSON": {
            "type": "text"
          },
          "refreshInterval": {
            "properties": {
              "display": {
                "type": "keyword"
              },
              "pause": {
                "type": "boolean"
              },
              "section": {
                "type": "integer"
              },
              "value": {
                "type": "integer"
              }
            }
          },
          "timeFrom": {
            "type": "keyword"
          },
          "timeRestore": {
            "type": "boolean"
          },
          "timeTo": {
            "type": "keyword"
          },
          "title": {
            "type": "text"
          },
          "uiStateJSON": {
            "type": "text"
          },
          "version": {
            "type": "integer"
          }
        }
      },
      "url": {
        "properties": {
          "accessCount": {
            "type": "long"
          },
          "accessDate": {
            "type": "date"
          },
          "createDate": {
            "type": "date"
          },
          "url": {
            "type": "text",
              "fields": {
              "keyword": {
                "type": "keyword",
                  "ignore_above": 2048
              }
            }
          }
        }
      },
      "server": {
        "properties": {
          "uuid": {
            "type": "keyword"
          }
        }
      },
      "timelion-sheet": {
        "properties": {
          "description": {
            "type": "text"
          },
          "hits": {
            "type": "integer"
          },
          "kibanaSavedObjectMeta": {
            "properties": {
              "searchSourceJSON": {
                "type": "text"
              }
            }
          },
          "timelion_chart_height": {
            "type": "integer"
          },
          "timelion_columns": {
            "type": "integer"
          },
          "timelion_interval": {
            "type": "keyword"
          },
          "timelion_other_interval": {
            "type": "keyword"
          },
          "timelion_rows": {
            "type": "integer"
          },
          "timelion_sheet": {
            "type": "text"
          },
          "title": {
            "type": "text"
          },
          "version": {
            "type": "integer"
          }
        }
      }
    }
  }'

  # reindex
  curl -XPOST "http://localhost:9200/_reindex" -H 'Content-Type: application/json' -d'
  {
    "source": {
      "index": ".kibana"
    },
    "dest": {
      "index": ".kibana-5.x"
    }
  }'
}
